@@ -56,12 +56,13 @@ class Request(object):
 		query_data		= parse_qs(env.get('QUERY_STRING'))	# query string
 		self.callback	= query_data.pop("callback") if "callback" in query_data else None
 		self.query_data	= query_data
+		self.form_vals	= {}
 		self.form_data	= {}
 		self.method		= env.get('REQUEST_METHOD')
 
 		if self.method == 'POST':
-			form_vals		= env['wsgi.input'].read()
-			self.form_data	= parse_qs(form_vals)
+			self.form_vals	= env['wsgi.input'].read()
+			self.form_data	= parse_qs(self.form_vals)
 
 		self._headers	= {}
 		self._cookies	= []
