@@ -34,6 +34,9 @@ class Routes(object):
 
 		def decorator(handler):
 			def wrapper(req, **kwargs):
+				h, v = content_type
+				req.add_header(h, v)
+
 				if self._setup_func:
 					self._setup_func(req)
 				if setup_func:
