@@ -12,8 +12,6 @@
 """
 
 from BaseHTTPServer import BaseHTTPRequestHandler
-from collections import namedtuple
-from urlparse import parse_qs
 from datetime import datetime
 from traceback import print_exc
 from sys import stderr, stdout, exc_info
@@ -27,11 +25,9 @@ from gevent.pywsgi import WSGIServer as pywsgi_server
 from gevent.wsgi import WSGIServer as wsgi_server
 #from gevent.pool import Pool
 
-from content_types import *
-
-'''_______________________________variables__________________________________'''
-
-status_codes = BaseHTTPRequestHandler.responses
+from . content_types import *
+from . request import Request
+from . exceptions import HttpError, ResponseStartedError
 
 '''________________________________helpers___________________________________'''
 
@@ -48,50 +44,42 @@ def _body_wrap(content_type):
 	else:
 		return normal
 
-def _poll(body, content_type):
+def _err():
+	err = None
+	info = exc_info()
+	if info and len(info) > 1:
+		err = info[1]
+	return err
+	
+def _poll(req, content_type, body):
 	fn = _body_wrap(content_type)
-	for b in body:
-		try:
-			yield fn(b)
-		except:
-			err = exc_info()[1]
-			log_error(err)
-			yield ''
+	first = ''
+	
+	# need to pull the first result to catch any headers that have been set
+	try: first = fn(body.next())
+	except:
+		log_error(_err())
+		req.start_response(code=500)
+	else:
+		req.start_response()
+		yield first
+
+		for b in body:
+			try:	yield fn(b)
+			except:	log_error(_err())
 
 def log_error(err):
-	'''Logs the given exception, and it's corresponding stacktrace'''
+	""" Logs the given exception, and it's corresponding stacktrace """
 	# TODO(jwilson): Implement logging to output to a real location
 	#				 OR if using *nix just redirect stderr to desired location
 	#				 and nothing below needs to change.
 	stderr.write('[Error] - %s\n'% (err,))
 	print_exc(6)
 
-def status(code):
-	''' Get the HTTP status for the given code '''
-	return "{0} {1}".format(code, status_codes[200][0])
-
-def parse_qd(env):
-	''' Parses either the query-string, or post data, and returns a dictionary
-		of key/list
-	'''
-	method = env.get('REQUEST_METHOD')
-	key = 'qs' if method == 'GET' else 'post_data'
-
-	return env.get(key)
-
-def parse_vals(env, *args):
-	''' Parses either the query-string, or post data, and returns the
-		values as a tuple
-	'''
-	data = parse_qd(env)
-	default = [None,]
-
-	return tuple([data.get(key, default)[0] for key in args])
-
 def get_json(body, callback=None):
-	''' Converts the `body` to JSON, except when `body` is a str and returns the
+	""" Converts the `body` to JSON, except when `body` is a str and returns the
 		result, wrapping it in the callback when provided.
-	'''
+	"""
 	result, err = '', None
 
 	try:
@@ -107,64 +95,61 @@ def get_json(body, callback=None):
 
 	return result, err
 
-def error_handler(env, resp, code, err=None, content_type=TEXT.HTML):
-	''' Handler used to log the given exception, if given, and return
+def error_handler(req, code, err=None, content_type=TEXT.HTML):
+	""" Handler used to log the given exception, if given, and return
 		an error message to be sent to the response.
-	'''
+	"""
 
 	if err is not None:
 		log_error(err)
 
-	resp(status(code), content_type)
+	req.start_response(code=code)
 
 	if content_type == APPLICATION.JSON:
 		return '{{ "error": {0} }}'.format(code)
 	else:
 		return 'error: {0}'.format(code)
 
-def handle_route(env, resp, route, kwargs):
-	''' Handler used to process routes.
+def handle_route(req, route, kwargs):
+	""" Handler used to process routes.
 		Supports: Yielding (polling) of routes
 				  JSON dumping when `Content-Type` is JSON
 				  gevent Queues
-	'''
+	"""
 
-	query_data = parse_qs(env.get('QUERY_STRING'))	# query string
-	callback = query_data.pop('callback', None)		# callback values
-	env['qs'] = query_data
-
-	if env.get('REQUEST_METHOD') == 'POST':
-		form_vals = env['wsgi.input'].read()
-		env['post_data'] = parse_qs(form_vals)
-	
 	try:
-		body = route(env, resp, **kwargs)
+		body = route(req, **kwargs)
+	except ResponseStartedError, err:
+		log_error(err)
+	except HttpError, err:
+		return error_handler(req, err.code, None, route.content_type)
 	except:
-		e = exc_info()[1]
-		return error_handler(env, resp, 500, e, route.content_type)
+		return error_handler(req, 500, _err(), route.content_type)
 	else:
 		if isinstance(body, GeneratorType):
-			body = _poll(body, route.content_type)	# get generator
+			return _poll(req, route.content_type, body)
 		elif route.content_type is APPLICATION.JSON:
-			body, err = get_json(body, callback)	# get content
+			body, err = get_json(body, req.callback)# get content
 			if err:									# error immediatly
-				return error_handler(env, resp, 500, err, route.content_type)
+				return error_handler(req, 500, err, route.content_type)
 
-		resp(status(200), route.content_type)		# success
+		req.start_response()
 		return body
 
 def get_handler(routes):
+	""" Gets a WSGI compatible handler """
 	def handler(env, resp):
 		path = env.get('PATH_INFO')
 		method = env.get('REQUEST_METHOD')
 		route, kwargs = routes.get(path)
+		req = Request(env, resp)
 
 		if not route:
-			return error_handler(env, resp, 404)
+			return error_handler(req, 404)
 		elif method != route.method and method not in route.methods:
-			return error_handler(env, resp, 501, content_type=route.content_type)
+			return error_handler(req, 501, content_type=route.content_type)
 		else:
-			return handle_route(env, resp, route, kwargs)
+			return handle_route(req, route, kwargs)
 	return handler
 
 '''________________________________server____________________________________'''
@@ -194,13 +179,16 @@ class WSGIServer:
 				.format(datetime.today(), self._addr, self._port))
 
 	def start(self):
+		""" Starts serving """
 		self._print_start_msg()
 		self._server.start()
 
 	def stop(self):
+		""" Stops serving """
 		self._server.stop()
 
 	def serve_forever(self):
+		""" Listens for incoming requests """
 		self._print_start_msg()
 		self._server.serve_forever()
 
